@@ -1,5 +1,5 @@
 /* setlocale() function that respects the locale chosen by the user.
-   Copyright (C) 2009, 2011, 2013, 2018-2019, 2022 Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011, 2013, 2018-2019, 2022-2023 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2009.
 
    This program is free software: you can redistribute it and/or modify
@@ -33,15 +33,28 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* When building a DLL, we must export some functions.  Note that because
-   the functions are only defined for binary backward compatibility, we
-   don't need to use __declspec(dllimport) in any case.  */
-#if HAVE_VISIBILITY && BUILDING_DLL
-# define DLL_EXPORTED __attribute__((__visibility__("default")))
-#elif defined _MSC_VER && BUILDING_DLL
-# define DLL_EXPORTED __declspec(dllexport)
+/* When building a shared library, we must export some functions.
+   Note that because this is a .c file, not a .h file, we don't need to use
+   __declspec(dllimport) in any case.  */
+#if HAVE_VISIBILITY && BUILDING_LIBRARY
+# define SHLIB_EXPORTED __attribute__((__visibility__("default")))
+#elif defined _MSC_VER && BUILDING_LIBRARY
+/* When building with MSVC, exporting a symbol means that the object file
+   contains a "linker directive" of the form /EXPORT:symbol.  This can be
+   inspected through the "objdump -s --section=.drectve FILE" or
+   "dumpbin /directives FILE" commands.
+   The symbols from this file should be exported if and only if the object
+   file gets included in a DLL.  Libtool, on Windows platforms, defines
+   the C macro DLL_EXPORT (together with PIC) when compiling for a shared
+   library (called DLL under Windows) and does not define it when compiling
+   an object file meant to be linked statically into some executable.  */
+# if defined DLL_EXPORT
+#  define SHLIB_EXPORTED __declspec(dllexport)
+# else
+#  define SHLIB_EXPORTED
+# endif
 #else
-# define DLL_EXPORTED
+# define SHLIB_EXPORTED
 #endif
 
 #include "localename.h"
@@ -1393,7 +1406,7 @@ get_main_locale_with_same_territory (const char *locale)
 
 # endif
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 char *
 libintl_setlocale (int category, const char *locale)
 {
@@ -1666,7 +1679,7 @@ libintl_setlocale (int category, const char *locale)
 
 # if HAVE_NEWLOCALE
 
-DLL_EXPORTED
+SHLIB_EXPORTED
 locale_t
 libintl_newlocale (int category_mask, const char *locale, locale_t base)
 {
