@@ -37,12 +37,13 @@
 #include "fatal-signal.h"
 #include "filename.h"
 #include "findprog.h"
+#include "windows-path.h"
 #include "unistd-safer.h"
 #include "wait-process.h"
 #include "xalloc.h"
 #include "gettext.h"
 
-#define _(str) gettext (str)
+#define _(msgid) dgettext ("gnulib", msgid)
 
 
 /* Choice of implementation for native Windows.
@@ -138,6 +139,7 @@ static pid_t
 create_pipe (const char *progname,
              const char *prog_path,
              const char * const *prog_argv,
+             const char * const *dll_dirs,
              const char *directory,
              bool pipe_stdin, bool pipe_stdout,
              const char *prog_stdin, const char *prog_stdout,
@@ -313,7 +315,8 @@ create_pipe (const char *progname,
         (HANDLE) _get_osfhandle (null_stderr ? nulloutfd : STDERR_FILENO);
 
       child = spawnpvech (P_NOWAIT, prog_path, argv + 1,
-                          (const char * const *) environ, directory,
+                          (const char * const *) environ, dll_dirs,
+                          directory,
                           stdin_handle, stdout_handle, stderr_handle);
 #  if 0 /* Executing arbitrary files as shell scripts is unsecure.  */
       if (child == -1 && errno == ENOEXEC)
@@ -323,7 +326,8 @@ create_pipe (const char *progname,
              a hidden element "sh.exe" to argv.  */
           argv[1] = prog_path;
           child = spawnpvech (P_NOWAIT, argv[0], argv,
-                              (const char * const *) environ, directory,
+                              (const char * const *) environ, dll_dirs,
+                              directory,
                               stdin_handle, stdout_handle, stderr_handle);
         }
 #  endif
@@ -356,62 +360,43 @@ create_pipe (const char *progname,
     }
 
 # else /* __KLIBC__ */
-  if (!(directory == NULL || strcmp (directory, ".") == 0))
-    {
-      /* A directory argument is not supported in this implementation.  */
-      saved_errno = EINVAL;
-      goto fail_with_saved_errno;
-    }
-
-  int orig_stdin;
-  int orig_stdout;
-  int orig_stderr;
-
-  /* Save standard file handles of parent process.  */
-  if (pipe_stdin || prog_stdin != NULL)
-    orig_stdin = dup_safer_noinherit (STDIN_FILENO);
-  if (pipe_stdout || prog_stdout != NULL)
-    orig_stdout = dup_safer_noinherit (STDOUT_FILENO);
-  if (null_stderr)
-    orig_stderr = dup_safer_noinherit (STDERR_FILENO);
+  int new_stdin = STDIN_FILENO;
+  int new_stdout = STDOUT_FILENO;
+  int new_stderr = STDERR_FILENO;
 
   /* Create standard file handles of child process.  */
   nulloutfd = -1;
   stdinfd = -1;
   stdoutfd = -1;
-  if ((!pipe_stdin || dup2 (ofd[0], STDIN_FILENO) >= 0)
-      && (!pipe_stdout || dup2 (ifd[1], STDOUT_FILENO) >= 0)
+  if ((!pipe_stdin || (new_stdin = ofd[0]) >= 0)
+      && (!pipe_stdout || (new_stdout = ifd[1]) >= 0)
       && (!null_stderr
-          || ((nulloutfd = open ("NUL", O_RDWR, 0)) >= 0
-              && (nulloutfd == STDERR_FILENO
-                  || (dup2 (nulloutfd, STDERR_FILENO) >= 0
-                      && close (nulloutfd) >= 0))))
+          || ((nulloutfd = open ("NUL", O_RDWR | O_CLOEXEC, 0)) >= 0
+              && (new_stderr = nulloutfd) >= 0))
       && (pipe_stdin
           || prog_stdin == NULL
-          || ((stdinfd = open (prog_stdin, O_RDONLY, 0)) >= 0
-              && (stdinfd == STDIN_FILENO
-                  || (dup2 (stdinfd, STDIN_FILENO) >= 0
-                      && close (stdinfd) >= 0))))
+          || ((stdinfd = open (prog_stdin, O_RDONLY | O_CLOEXEC, 0)) >= 0
+              && (new_stdin = stdinfd) >= 0))
       && (pipe_stdout
           || prog_stdout == NULL
-          || ((stdoutfd = open (prog_stdout, O_WRONLY, 0)) >= 0
-              && (stdoutfd == STDOUT_FILENO
-                  || (dup2 (stdoutfd, STDOUT_FILENO) >= 0
-                      && close (stdoutfd) >= 0)))))
+          || ((stdoutfd = open (prog_stdout, O_WRONLY | O_CLOEXEC, 0)) >= 0
+              && (new_stdout = stdoutfd) >= 0)))
     /* The child process doesn't inherit ifd[0], ifd[1], ofd[0], ofd[1],
        but it inherits all open()ed or dup2()ed file handles (which is what
        we want in the case of STD*_FILENO).  */
     {
-      child = _spawnvpe (P_NOWAIT, prog_path, argv + 1,
-                         (const char **) environ);
+      child = spawnpvech (P_NOWAIT, prog_path, argv + 1,
+                          (const char **) environ, directory,
+                          new_stdin, new_stdout, new_stderr);
 #  if 0 /* Executing arbitrary files as shell scripts is unsecure.  */
       if (child == -1 && errno == ENOEXEC)
         {
           /* prog is not a native executable.  Try to execute it as a
              shell script.  Note that prepare_spawn() has already prepended
              a hidden element "sh.exe" to argv.  */
-          child = _spawnvpe (P_NOWAIT, argv[0], argv,
-                             (const char **) environ);
+          child = spawnpvech (P_NOWAIT, argv[0], argv,
+                              (const char **) environ, directory,
+                              new_stdin, new_stdout, new_stderr);
         }
 #  endif
     }
@@ -424,14 +409,6 @@ create_pipe (const char *progname,
   if (nulloutfd >= 0)
     close (nulloutfd);
 
-  /* Restore standard file handles of parent process.  */
-  if (null_stderr)
-    undup_safer_noinherit (orig_stderr, STDERR_FILENO);
-  if (pipe_stdout || prog_stdout != NULL)
-    undup_safer_noinherit (orig_stdout, STDOUT_FILENO);
-  if (pipe_stdin || prog_stdin != NULL)
-    undup_safer_noinherit (orig_stdin, STDIN_FILENO);
-
   if (pipe_stdin)
     close (ofd[0]);
   if (pipe_stdout)
@@ -460,6 +437,8 @@ create_pipe (const char *progname,
 #else
 
   /* Unix API.  */
+  char **child_environ;
+  char **malloced_environ;
   sigset_t blocked_signals;
   posix_spawn_file_actions_t actions;
   bool actions_allocated;
@@ -468,6 +447,18 @@ create_pipe (const char *progname,
   int err;
   pid_t child;
 
+  child_environ = environ;
+  malloced_environ = NULL;
+# if defined _WIN32 || defined __CYGWIN__
+  if (dll_dirs != NULL && dll_dirs[0] != NULL)
+    {
+      malloced_environ = extended_environ (dll_dirs);
+      if (malloced_environ == NULL)
+        goto fail_with_errno;
+      child_environ = malloced_environ;
+    }
+# endif
+
   if (slave_process)
     {
       sigprocmask (SIG_SETMASK, NULL, &blocked_signals);
@@ -526,24 +517,26 @@ create_pipe (const char *progname,
 # if defined _WIN32 && !defined __CYGWIN__
                       (err = posix_spawnattr_setpgroup (&attrs, 0)) != 0
                       || (err = posix_spawnattr_setflags (&attrs,
-                                                         POSIX_SPAWN_SETPGROUP))
+                                                          POSIX_SPAWN_SETPGROUP))
                          != 0
 # else
                       (err = posix_spawnattr_setsigmask (&attrs,
                                                          &blocked_signals))
                       != 0
                       || (err = posix_spawnattr_setflags (&attrs,
-                                                        POSIX_SPAWN_SETSIGMASK))
+                                                          POSIX_SPAWN_SETSIGMASK))
                          != 0
 # endif
              )   )   )
           || (err = (directory != NULL
                      ? posix_spawn (&child, prog_path, &actions,
                                     attrs_allocated ? &attrs : NULL,
-                                    (char * const *) prog_argv, environ)
+                                    (char * const *) prog_argv,
+                                    child_environ)
                      : posix_spawnp (&child, prog_path, &actions,
                                      attrs_allocated ? &attrs : NULL,
-                                     (char * const *) prog_argv, environ)))
+                                     (char * const *) prog_argv,
+                                     child_environ)))
              != 0))
     {
       if (actions_allocated)
@@ -552,6 +545,11 @@ create_pipe (const char *progname,
         posix_spawnattr_destroy (&attrs);
       if (slave_process)
         unblock_fatal_signals ();
+      if (malloced_environ != NULL)
+        {
+          free (malloced_environ[0]);
+          free (malloced_environ);
+        }
       if (pipe_stdout)
         {
           close (ifd[0]);
@@ -574,6 +572,11 @@ create_pipe (const char *progname,
       register_slave_subprocess (child);
       unblock_fatal_signals ();
     }
+  if (malloced_environ != NULL)
+    {
+      free (malloced_environ[0]);
+      free (malloced_environ);
+    }
   if (pipe_stdin)
     close (ofd[0]);
   if (pipe_stdout)
@@ -609,12 +612,14 @@ create_pipe (const char *progname,
 pid_t
 create_pipe_bidi (const char *progname,
                   const char *prog_path, const char * const *prog_argv,
+                  const char * const *dll_dirs,
                   const char *directory,
                   bool null_stderr,
                   bool slave_process, bool exit_on_error,
                   int fd[2])
 {
-  pid_t result = create_pipe (progname, prog_path, prog_argv, directory,
+  pid_t result = create_pipe (progname, prog_path, prog_argv, dll_dirs,
+                              directory,
                               true, true, NULL, NULL,
                               null_stderr, slave_process, exit_on_error,
                               fd);
@@ -631,13 +636,15 @@ create_pipe_bidi (const char *progname,
 pid_t
 create_pipe_in (const char *progname,
                 const char *prog_path, const char * const *prog_argv,
+                const char * const *dll_dirs,
                 const char *directory,
                 const char *prog_stdin, bool null_stderr,
                 bool slave_process, bool exit_on_error,
                 int fd[1])
 {
   int iofd[2];
-  pid_t result = create_pipe (progname, prog_path, prog_argv, directory,
+  pid_t result = create_pipe (progname, prog_path, prog_argv, dll_dirs,
+                              directory,
                               false, true, prog_stdin, NULL,
                               null_stderr, slave_process, exit_on_error,
                               iofd);
@@ -656,13 +663,15 @@ create_pipe_in (const char *progname,
 pid_t
 create_pipe_out (const char *progname,
                  const char *prog_path, const char * const *prog_argv,
+                 const char * const *dll_dirs,
                  const char *directory,
                  const char *prog_stdout, bool null_stderr,
                  bool slave_process, bool exit_on_error,
                  int fd[1])
 {
   int iofd[2];
-  pid_t result = create_pipe (progname, prog_path, prog_argv, directory,
+  pid_t result = create_pipe (progname, prog_path, prog_argv, dll_dirs,
+                              directory,
                               true, false, NULL, prog_stdout,
                               null_stderr, slave_process, exit_on_error,
                               iofd);
