@@ -1,9 +1,9 @@
 /* Test of opening a file descriptor.
-   Copyright (C) 2007-2020 Free Software Foundation, Inc.
+   Copyright (C) 2007-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -24,7 +24,6 @@
 SIGNATURE_CHECK (open, int, (char const *, int, ...));
 
 #include <errno.h>
-#include <stdbool.h>
 #include <stdio.h>
 #include <unistd.h>
 
@@ -37,5 +36,6 @@ SIGNATURE_CHECK (open, int, (char const *, int, ...));
 int
 main (void)
 {
-  return test_open (open, true);
+  int result = test_open (open, true);
+  return (result ? result : test_exit_status);
 }
