@@ -1,9 +1,9 @@
 /* Test posix_spawn_file_actions_addchdir() function.
-   Copyright (C) 2018-2022 Free Software Foundation, Inc.
+   Copyright (C) 2018-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -40,5 +40,5 @@ main (void)
 
   posix_spawn_file_actions_destroy (&actions);
 
-  return 0;
+  return test_exit_status;
 }
