@@ -1,9 +1,9 @@
 /* Test of xmemdup0() function.
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2024 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
+   the Free Software Foundation, either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -74,5 +74,5 @@ main (int argc, char **argv)
     free (result);
   }
 
-  return 0;
+  return test_exit_status;
 }
