@@ -1,5 +1,5 @@
 /* C format strings.
-   Copyright (C) 2001-2004, 2006-2007, 2009-2010, 2019 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004, 2006-2007, 2009-2010, 2019, 2023 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -38,6 +38,9 @@
 #define INVALID_ANGLE_BRACKET(directive_number) \
   xasprintf (_("In the directive number %u, the token after '<' is not followed by '>'."), directive_number)
 
+#define INVALID_SIZE_SPECIFIER(directive_number) \
+  xasprintf (_("In the directive number %u, the argument size specifier is invalid."), directive_number)
+
 #define INVALID_IGNORED_ARGUMENT(referenced_arg, ignored_arg) \
   xasprintf (_("The string refers to argument number %u but ignores argument number %u."), referenced_arg, ignored_arg)
 
@@ -120,7 +123,7 @@ format_get_number_of_directives (void *descr)
 
 static bool
 format_check (void *msgid_descr, void *msgstr_descr, bool equality,
-              formatstring_error_logger_t error_logger,
+              formatstring_error_logger_t error_logger, void *error_logger_data,
               const char *pretty_msgid, const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
@@ -134,7 +137,8 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
       : spec1->unnumbered_arg_count < spec2->unnumbered_arg_count)
     {
       if (error_logger)
-        error_logger (_("number of format specifications in '%s' and '%s' does not match"),
+        error_logger (error_logger_data,
+                      _("number of format specifications in '%s' and '%s' does not match"),
                       pretty_msgid, pretty_msgstr);
       err = true;
     }
@@ -143,7 +147,8 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
       if (spec1->unnumbered[i].type != spec2->unnumbered[i].type)
         {
           if (error_logger)
-            error_logger (_("format specifications in '%s' and '%s' for argument %u are not the same"),
+            error_logger (error_logger_data,
+                          _("format specifications in '%s' and '%s' for argument %u are not the same"),
                           pretty_msgid, pretty_msgstr, i + 1);
           err = true;
         }
