@@ -1,5 +1,5 @@
 /* KUIT (KDE User Interface Text) format strings.
-   Copyright (C) 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2015-2024 Free Software Foundation, Inc.
    Written by Daiki Ueno <ueno@gnu.org>, 2015.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -53,7 +53,7 @@
 
 /* KUIT (KDE User Interface Text) is an XML-like markup which augments
    translatable strings with semantic information:
-   http://api.kde.org/frameworks-api/frameworks5-apidocs/ki18n/html/prg_guide.html#kuit_markup
+   https://api.kde.org/frameworks/ki18n/html/prg_guide.html#kuit_markup
    KUIT can be seen as a fragment of a well-formed XML document,
    except that it allows '&' as a Qt accelerator marker and '%' as a
    format directive.  */
@@ -73,7 +73,7 @@ struct char_range
 };
 
 /* Character ranges for NameStartChar defined in:
-   http://www.w3.org/TR/REC-xml/#NT-NameStartChar  */
+   https://www.w3.org/TR/REC-xml/#NT-NameStartChar  */
 static const struct char_range name_chars1[] =
   {
     { ':', ':' },
@@ -95,7 +95,7 @@ static const struct char_range name_chars1[] =
   };
 
 /* Character ranges for NameChar, excluding NameStartChar:
-   http://www.w3.org/TR/REC-xml/#NT-NameChar  */
+   https://www.w3.org/TR/REC-xml/#NT-NameChar  */
 static const struct char_range name_chars2[] =
   {
     { '-', '-' },
@@ -239,7 +239,7 @@ format_parse (const char *format, bool translated, char *fdi,
                            | XML_PARSE_NOBLANKS);
       if (doc == NULL)
         {
-          xmlError *err = xmlGetLastError ();
+          const xmlError *err = xmlGetLastError ();
           *invalid_reason =
             xasprintf (_("error while parsing: %s"),
                        err->message);
@@ -251,7 +251,7 @@ format_parse (const char *format, bool translated, char *fdi,
       free (buffer);
       xmlFreeDoc (doc);
     }
-#elif FORMAT_KDE_KUIT_FALLBACK_MARKUP
+#elif FORMAT_KDE_KUIT_USE_FALLBACK_MARKUP
     {
       markup_parser_ty parser;
       markup_parse_context_ty *context;
@@ -283,6 +283,7 @@ format_parse (const char *format, bool translated, char *fdi,
     }
 #else
     /* No support for XML.  */
+    free (buffer);
 #endif
 
   spec.base = formatstring_kde.parse (format, translated, fdi, invalid_reason);
@@ -311,14 +312,14 @@ format_get_number_of_directives (void *descr)
 
 static bool
 format_check (void *msgid_descr, void *msgstr_descr, bool equality,
-              formatstring_error_logger_t error_logger,
+              formatstring_error_logger_t error_logger, void *error_logger_data,
               const char *pretty_msgid, const char *pretty_msgstr)
 {
   struct spec *msgid_spec = msgid_descr;
   struct spec *msgstr_spec = msgstr_descr;
 
   return formatstring_kde.check (msgid_spec->base, msgstr_spec->base, equality,
-                                 error_logger,
+                                 error_logger, error_logger_data,
                                  pretty_msgid, pretty_msgstr);
 }
 
@@ -404,7 +405,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../intl -DHAVE_CONFIG_H -DTEST format-kde-kuit.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-kde-kuit.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
