@@ -1,5 +1,5 @@
 /* Reading Desktop Entry files.
-   Copyright (C) 1995-1998, 2000-2003, 2005-2006, 2008-2009, 2014-2019, 2023 Free Software Foundation, Inc.
+   Copyright (C) 1995-2024 Free Software Foundation, Inc.
    This file was written by Daiki Ueno <ueno@gnu.org>.
 
    This program is free software: you can redistribute it and/or modify
@@ -22,8 +22,6 @@
 /* Specification.  */
 #include "read-desktop.h"
 
-#include "xalloc.h"
-
 #include <assert.h>
 #include <errno.h>
 #include <stdbool.h>
@@ -31,12 +29,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "error.h"
-#include "error-progname.h"
+#include <error.h>
 #include "xalloc.h"
 #include "xvasprintf.h"
+#include "string-buffer.h"
 #include "c-ctype.h"
-#include "po-lex.h"
 #include "po-xerror.h"
 #include "gettext.h"
 
@@ -101,7 +98,7 @@ desktop_reader_handle_blank (desktop_reader_ty *reader, const char *s)
 static const char *real_file_name;
 
 /* File name and line number.  */
-extern lex_pos_ty gram_pos;
+static lex_pos_ty pos;
 
 /* The input file stream.  */
 static FILE *fp;
@@ -164,7 +161,7 @@ phase2_getc ()
     }
 
   if (c == '\n')
-    gram_pos.line_number++;
+    pos.line_number++;
 
   return c;
 }
@@ -173,7 +170,7 @@ static void
 phase2_ungetc (int c)
 {
   if (c == '\n')
-    --gram_pos.line_number;
+    --pos.line_number;
   if (c != EOF)
     phase2_pushback[phase2_pushback_length++] = c;
 }
@@ -198,6 +195,7 @@ struct token_ty
   char *string;
   const char *value;
   const char *locale;
+  size_t lineno;
 };
 
 /* Free the memory pointed to by a 'struct token_ty'.  */
@@ -212,24 +210,12 @@ free_token (token_ty *tp)
 static void
 desktop_lex (token_ty *tp)
 {
-  static char *buffer;
-  static size_t bufmax;
-  size_t bufpos;
+  struct string_buffer buffer;
 
 #undef APPEND
-#define APPEND(c)                               \
-  do                                            \
-    {                                           \
-      if (bufpos >= bufmax)                     \
-        {                                       \
-          bufmax += 100;                        \
-          buffer = xrealloc (buffer, bufmax);   \
-        }                                       \
-      buffer[bufpos++] = c;                     \
-    }                                           \
-  while (0)
+#define APPEND(c) sb_xappend1 (&buffer, (c))
 
-  bufpos = 0;
+  sb_init (&buffer);
   for (;;)
     {
       int c;
@@ -239,12 +225,14 @@ desktop_lex (token_ty *tp)
       switch (c)
         {
         case EOF:
+          sb_free (&buffer);
           tp->type = token_type_eof;
           return;
 
         case '[':
           {
             bool non_blank = false;
+            size_t non_blank_lineno = 0;
 
             for (;;)
               {
@@ -254,7 +242,7 @@ desktop_lex (token_ty *tp)
                 if (c == '\n')
                   {
                     po_xerror (PO_SEVERITY_WARNING, NULL,
-                               real_file_name, gram_pos.line_number, 0, false,
+                               real_file_name, pos.line_number - 1, 0, false,
                                _("unterminated group name"));
                     break;
                   }
@@ -271,15 +259,18 @@ desktop_lex (token_ty *tp)
                 if (c == EOF)
                   break;
                 if (!c_isspace (c))
-                  non_blank = true;
+                  {
+                    non_blank = true;
+                    non_blank_lineno = pos.line_number;
+                  }
               }
             if (non_blank)
               po_xerror (PO_SEVERITY_WARNING, NULL,
-                         real_file_name, gram_pos.line_number, 0, false,
+                         real_file_name, non_blank_lineno, 0, false,
                          _("invalid non-blank character"));
             APPEND (0);
             tp->type = token_type_group;
-            tp->string = xstrdup (buffer);
+            tp->string = sb_xdupfree_c (&buffer);
             return;
           }
 
@@ -295,7 +286,7 @@ desktop_lex (token_ty *tp)
               }
             APPEND (0);
             tp->type = token_type_comment;
-            tp->string = xstrdup (buffer);
+            tp->string = sb_xdupfree_c (&buffer);
             return;
           }
 
@@ -342,7 +333,7 @@ desktop_lex (token_ty *tp)
                     /* Finish the key part and start the locale part.  */
                     APPEND (0);
                     found_locale = true;
-                    locale_start = bufpos;
+                    locale_start = string_desc_length (sb_contents (&buffer));
 
                     for (;;)
                       {
@@ -362,6 +353,7 @@ desktop_lex (token_ty *tp)
             APPEND (0);
 
             /* Skip any space before '='.  */
+            size_t before_equals_lineno = pos.line_number;
             for (;;)
               {
                 c = phase2_getc ();
@@ -382,8 +374,10 @@ desktop_lex (token_ty *tp)
             if (c != '=')
               {
                 po_xerror (PO_SEVERITY_WARNING, NULL,
-                           real_file_name, gram_pos.line_number, 0, false,
-                           xasprintf (_("missing '=' after \"%s\""), buffer));
+                           real_file_name, before_equals_lineno, 0, false,
+                           xasprintf (_("missing '=' after \"%s\""),
+                                      sb_xcontents_c (&buffer)));
+                sb_free (&buffer);
                 for (;;)
                   {
                     c = phase2_getc ();
@@ -411,7 +405,8 @@ desktop_lex (token_ty *tp)
                 break;
               }
 
-            value_start = bufpos;
+            size_t before_value_lineno = pos.line_number;
+            value_start = string_desc_length (sb_contents (&buffer));
             for (;;)
               {
                 c = phase2_getc ();
@@ -420,15 +415,20 @@ desktop_lex (token_ty *tp)
                 APPEND (c);
               }
             APPEND (0);
+            char *buffer_contents = sb_xdupfree_c (&buffer);
             tp->type = token_type_pair;
-            tp->string = xmemdup (buffer, bufpos);
-            tp->locale = found_locale ? &buffer[locale_start] : NULL;
-            tp->value = &buffer[value_start];
+            tp->string = buffer_contents;
+            /* tp->locale and tp->value are live only as long as tp->string is
+               live.  */
+            tp->locale = found_locale ? &buffer_contents[locale_start] : NULL;
+            tp->value = &buffer_contents[value_start];
+            tp->lineno = before_value_lineno;
             return;
           }
         default:
           {
             bool non_blank = false;
+            size_t non_blank_lineno = 0;
 
             for (;;)
               {
@@ -436,7 +436,10 @@ desktop_lex (token_ty *tp)
                   break;
 
                 if (!c_isspace (c))
-                  non_blank = true;
+                  {
+                    non_blank = true;
+                    non_blank_lineno = pos.line_number;
+                  }
                 else
                   APPEND (c);
 
@@ -445,14 +448,15 @@ desktop_lex (token_ty *tp)
             if (non_blank)
               {
                 po_xerror (PO_SEVERITY_WARNING, NULL,
-                           real_file_name, gram_pos.line_number, 0, false,
+                           real_file_name, non_blank_lineno, 0, false,
                            _("invalid non-blank line"));
+                sb_free (&buffer);
                 tp->type = token_type_other;
                 return;
               }
             APPEND (0);
             tp->type = token_type_blank;
-            tp->string = xstrdup (buffer);
+            tp->string = sb_xdupfree_c (&buffer);
             return;
           }
         }
@@ -466,8 +470,8 @@ desktop_parse (desktop_reader_ty *reader, FILE *file,
 {
   fp = file;
   real_file_name = real_filename;
-  gram_pos.file_name = xstrdup (logical_filename);
-  gram_pos.line_number = 1;
+  pos.file_name = xstrdup (logical_filename);
+  pos.line_number = 1;
 
   for (;;)
     {
@@ -484,8 +488,14 @@ desktop_parse (desktop_reader_ty *reader, FILE *file,
           desktop_reader_handle_comment (reader, token.string);
           break;
         case token_type_pair:
-          desktop_reader_handle_pair (reader, &gram_pos,
-                                      token.string, token.locale, token.value);
+          {
+            lex_pos_ty token_pos;
+            token_pos.file_name = pos.file_name;
+            token_pos.line_number = token.lineno;
+
+            desktop_reader_handle_pair (reader, &token_pos,
+                                        token.string, token.locale, token.value);
+          }
           break;
         case token_type_blank:
           desktop_reader_handle_blank (reader, token.string);
@@ -499,7 +509,7 @@ desktop_parse (desktop_reader_ty *reader, FILE *file,
  out:
   fp = NULL;
   real_file_name = NULL;
-  gram_pos.line_number = 0;
+  pos.line_number = 0;
 }
 
 char *
