@@ -1,5 +1,5 @@
 /* xgettext Python backend.
-   Copyright (C) 2002-2003, 2005-2011, 2013-2014, 2018-2020 Free Software Foundation, Inc.
+   Copyright (C) 2002-2024 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
@@ -30,6 +30,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <error.h>
 #include "attribute.h"
 #include "message.h"
 #include "rc-str-list.h"
@@ -41,13 +42,14 @@
 #include "xg-arglist-callshape.h"
 #include "xg-arglist-parser.h"
 #include "xg-message.h"
-#include "error.h"
-#include "error-progname.h"
+#include "if-error.h"
+#include "xstrerror.h"
 #include "progname.h"
 #include "basename-lgpl.h"
 #include "xerror.h"
 #include "xvasprintf.h"
 #include "xalloc.h"
+#include "string-buffer.h"
 #include "c-strstr.h"
 #include "c-ctype.h"
 #include "po-charset.h"
@@ -65,8 +67,10 @@
 
 /* The Python syntax is defined in the Python Reference Manual
    /usr/share/doc/packages/python/html/ref/index.html.
-   See also Python-2.0/Parser/tokenizer.c, Python-2.0/Python/compile.c,
-   Python-2.0/Objects/unicodeobject.c.  */
+   See also Python-3.7.17/Parser/tokenizer.c, Python-3.7.17/Python/compile.c,
+   Python-3.7.17/Objects/bytesobject.c, Python-3.7.17/Objects/unicodeobject.c.
+   For the f-strings, refer to https://peps.python.org/pep-0498/
+   and https://docs.python.org/3/reference/lexical_analysis.html#literals .  */
 
 
 /* ====================== Keyword set customization.  ====================== */
@@ -196,7 +200,7 @@ phase0_getc ()
 }
 
 /* Supports only one pushback character, and not '\n'.  */
-static inline void
+MAYBE_UNUSED static inline void
 phase0_ungetc (int c)
 {
   if (c != EOF)
@@ -293,11 +297,14 @@ as specified in https://www.python.org/peps/pep-0263.html.\n")));
          interactive behaviour when fp is connected to an interactive tty.  */
       unsigned char buf[MAX_PHASE1_PUSHBACK];
       size_t bufcount;
-      int c = phase1_getc ();
-      if (c == EOF)
-        return UEOF;
-      buf[0] = (unsigned char) c;
-      bufcount = 1;
+
+      {
+        int c = phase1_getc ();
+        if (c == EOF)
+          return UEOF;
+        buf[0] = (unsigned char) c;
+        bufcount = 1;
+      }
 
       for (;;)
         {
@@ -352,8 +359,9 @@ comment as specified in https://www.python.org/peps/pep-0263.html.\n"),
                   buf[bufcount++] = (unsigned char) c;
                 }
               else
-                error (EXIT_FAILURE, errno, _("%s:%d: iconv failure"),
-                       real_file_name, line_number);
+                if_error (IF_SEVERITY_FATAL_ERROR,
+                          real_file_name, line_number, (size_t)(-1), false,
+                          "%s", xstrerror (_("iconv failure"), errno));
             }
           else
             {
@@ -611,16 +619,9 @@ set_current_file_source_encoding (const char *canon_encoding)
       && xgettext_current_file_source_encoding != po_charset_utf8)
     {
 #if HAVE_ICONV
-      iconv_t cd;
+      iconv_t cd =
+        iconv_open (po_charset_utf8, xgettext_current_file_source_encoding);
 
-      /* Avoid glibc-2.1 bug with EUC-KR.  */
-# if ((__GLIBC__ == 2 && __GLIBC_MINOR__ <= 1) && !defined __UCLIBC__) \
-     && !defined _LIBICONV_VERSION
-      if (strcmp (xgettext_current_file_source_encoding, "EUC-KR") == 0)
-        cd = (iconv_t)(-1);
-      else
-# endif
-      cd = iconv_open (po_charset_utf8, xgettext_current_file_source_encoding);
       if (cd == (iconv_t)(-1))
         error_at_line (EXIT_FAILURE, 0, logical_file_name, line_number - 1,
                        _("Cannot convert from \"%s\" to \"%s\". %s relies on iconv(), and iconv() does not support this conversion."),
@@ -770,6 +771,7 @@ phase3_ungetc (int c)
 /* Return value of phase7_getuc when EOF is reached.  */
 #define P7_EOF (-1)
 #define P7_STRING_END (-2)
+#define P7_498_START_OF_EXPRESSION (-3) /* { */
 
 /* Convert an UTF-16 or UTF-32 code point to a return value that can be
    distinguished from a single-byte return value.  */
@@ -796,6 +798,10 @@ enum token_type_ty
   token_type_lbracket,          /* [ */
   token_type_rbracket,          /* ] */
   token_type_string,            /* "abc", 'abc', """abc""", '''abc''' */
+  token_type_498,               /* f"abc", f'abc', f"""abc""", f'''abc''' */
+  token_type_l498,              /* left part of f-string: f"abc{, f'abc{, f"""abc{, f'''abc{ */
+  token_type_m498,              /* middle part of f-string: }abc{ */
+  token_type_r498,              /* right part of f-string: }abc", }abc', }abc""", }abc''' */
   token_type_symbol,            /* symbol, number */
   token_type_plus,              /* + */
   token_type_other              /* misc. operator */
@@ -806,9 +812,9 @@ typedef struct token_ty token_ty;
 struct token_ty
 {
   token_type_ty type;
-  char *string;                         /* for token_type_symbol */
-  mixed_string_ty *mixed_string;        /* for token_type_string */
-  refcounted_string_list_ty *comment;   /* for token_type_string */
+  char *string;                       /* for token_type_symbol */
+  mixed_string_ty *mixed_string;      /* for token_type_string, token_type_498 */
+  refcounted_string_list_ty *comment; /* for token_type_string, token_type_498 */
   int line_number;
 };
 
@@ -818,7 +824,7 @@ free_token (token_ty *tp)
 {
   if (tp->type == token_type_symbol)
     free (tp->string);
-  if (tp->type == token_type_string)
+  if (tp->type == token_type_string || tp->type == token_type_498)
     {
       mixed_string_free (tp->mixed_string);
       drop_reference (tp->comment);
@@ -826,12 +832,13 @@ free_token (token_ty *tp)
 }
 
 
-/* There are two different input syntaxes for strings, "abc" and r"abc",
-   and two different input syntaxes for Unicode strings, u"abc" and ur"abc".
+/* There are two different input syntaxes for byte strings, b"abc" and br"abc",
+   and two different input syntaxes for Unicode strings, u"abc" and ur"abc";
+   the 'u' may be omitted.
    Which escape sequences are understood, i.e. what is interpreted specially
    after backslash?
-    "abc"     \<nl> \\ \' \" \a\b\f\n\r\t\v \ooo \xnn
-    r"abc"
+    b"abc"    \<nl> \\ \' \" \a\b\f\n\r\t\v \ooo \xnn
+    br"abc"
     u"abc"    \<nl> \\ \' \" \a\b\f\n\r\t\v \ooo \xnn \unnnn \Unnnnnnnn \N{...}
     ur"abc"                                           \unnnn
    The \unnnn values are UTF-16 values; a single \Unnnnnnnn can expand to two
@@ -842,6 +849,7 @@ free_token (token_ty *tp)
 static int
 phase7_getuc (int quote_char,
               bool triple, bool interpret_ansic, bool interpret_unicode,
+              bool f_string,
               unsigned int *backslash_counter)
 {
   int c;
@@ -889,13 +897,31 @@ phase7_getuc (int quote_char,
               return UNICODE ('\n');
             }
           phase2_ungetc (c);
-          error_with_progname = false;
-          error (0, 0, _("%s:%d: warning: unterminated string"),
-                 logical_file_name, line_number);
-          error_with_progname = true;
+          if_error (IF_SEVERITY_WARNING,
+                    logical_file_name, line_number, (size_t)(-1), false,
+                    _("unterminated string"));
           return P7_STRING_END;
         }
 
+      if (f_string)
+        {
+          if (c == '{')
+            {
+              int c1 = phase2_getc ();
+              if (c1 == '{')
+                return UNICODE ('{');
+              phase2_ungetc (c1);
+              return P7_498_START_OF_EXPRESSION;
+            }
+          if (c == '}')
+            {
+              int c1 = phase2_getc ();
+              if (c1 == '}')
+                return UNICODE ('}');
+              phase2_ungetc (c1);
+            }
+        }
+
       if (c != '\\')
         {
           *backslash_counter = 0;
@@ -974,6 +1000,11 @@ phase7_getuc (int quote_char,
                     phase2_ungetc (c);
                 }
               *backslash_counter = 0;
+              /* <https://docs.python.org/3.12/reference/lexical_analysis.html#escape-sequences>
+                 says: "In a bytes literal, hexadecimal and octal escapes denote
+                        the byte with the given value.
+                        In a string literal, these escapes denote a Unicode
+                        character with the given value."  */
               if (interpret_unicode)
                 return UNICODE (n);
               else
@@ -1011,6 +1042,12 @@ phase7_getuc (int quote_char,
                     {
                       int n = (n1 << 4) + n2;
                       *backslash_counter = 0;
+                      /* <https://docs.python.org/3.12/reference/lexical_analysis.html#escape-sequences>
+                         says:
+                         "In a bytes literal, hexadecimal and octal escapes denote
+                          the byte with the given value.
+                          In a string literal, these escapes denote a Unicode
+                          character with the given value."  */
                       if (interpret_unicode)
                         return UNICODE (n);
                       else
@@ -1026,7 +1063,7 @@ phase7_getuc (int quote_char,
             }
           }
 
-      if (interpret_unicode)
+      if (interpret_ansic && interpret_unicode)
         {
           if (c == 'u')
             {
@@ -1060,98 +1097,94 @@ phase7_getuc (int quote_char,
               return UNICODE (n);
             }
 
-          if (interpret_ansic)
+          if (c == 'U')
             {
-              if (c == 'U')
+              unsigned char buf[8];
+              unsigned int n = 0;
+              int i;
+
+              for (i = 0; i < 8; i++)
                 {
-                  unsigned char buf[8];
-                  unsigned int n = 0;
-                  int i;
+                  int c1 = phase2_getc ();
 
-                  for (i = 0; i < 8; i++)
+                  if (c1 >= '0' && c1 <= '9')
+                    n = (n << 4) + (c1 - '0');
+                  else if (c1 >= 'A' && c1 <= 'F')
+                    n = (n << 4) + (c1 - 'A' + 10);
+                  else if (c1 >= 'a' && c1 <= 'f')
+                    n = (n << 4) + (c1 - 'a' + 10);
+                  else
                     {
-                      int c1 = phase2_getc ();
+                      phase2_ungetc (c1);
+                      while (--i >= 0)
+                        phase2_ungetc (buf[i]);
+                      phase2_ungetc (c);
+                      ++*backslash_counter;
+                      return UNICODE ('\\');
+                    }
 
-                      if (c1 >= '0' && c1 <= '9')
-                        n = (n << 4) + (c1 - '0');
-                      else if (c1 >= 'A' && c1 <= 'F')
-                        n = (n << 4) + (c1 - 'A' + 10);
-                      else if (c1 >= 'a' && c1 <= 'f')
-                        n = (n << 4) + (c1 - 'a' + 10);
-                      else
+                  buf[i] = c1;
+                }
+              if (n < 0x110000)
+                {
+                  *backslash_counter = 0;
+                  return UNICODE (n);
+                }
+
+              if_error (IF_SEVERITY_WARNING,
+                        logical_file_name, line_number, (size_t)(-1), false,
+                        _("invalid Unicode character"));
+
+              while (--i >= 0)
+                phase2_ungetc (buf[i]);
+              phase2_ungetc (c);
+              ++*backslash_counter;
+              return UNICODE ('\\');
+            }
+
+          if (c == 'N')
+            {
+              int c1 = phase2_getc ();
+              if (c1 == '{')
+                {
+                  unsigned char buf[UNINAME_MAX + 1];
+                  int i;
+                  unsigned int n;
+
+                  for (i = 0; i < UNINAME_MAX; i++)
+                    {
+                      int c2 = phase2_getc ();
+                      if (!(c2 >= ' ' && c2 <= '~'))
                         {
-                          phase2_ungetc (c1);
+                          phase2_ungetc (c2);
                           while (--i >= 0)
                             phase2_ungetc (buf[i]);
+                          phase2_ungetc (c1);
                           phase2_ungetc (c);
                           ++*backslash_counter;
                           return UNICODE ('\\');
                         }
-
-                      buf[i] = c1;
+                      if (c2 == '}')
+                        break;
+                      buf[i] = c2;
                     }
-                  if (n < 0x110000)
+                  buf[i] = '\0';
+
+                  n = unicode_name_character ((char *) buf);
+                  if (n != UNINAME_INVALID)
                     {
                       *backslash_counter = 0;
                       return UNICODE (n);
                     }
 
-                  error_with_progname = false;
-                  error (0, 0, _("%s:%d: warning: invalid Unicode character"),
-                         logical_file_name, line_number);
-                  error_with_progname = true;
-
+                  phase2_ungetc ('}');
                   while (--i >= 0)
                     phase2_ungetc (buf[i]);
-                  phase2_ungetc (c);
-                  ++*backslash_counter;
-                  return UNICODE ('\\');
-                }
-
-              if (c == 'N')
-                {
-                  int c1 = phase2_getc ();
-                  if (c1 == '{')
-                    {
-                      unsigned char buf[UNINAME_MAX + 1];
-                      int i;
-                      unsigned int n;
-
-                      for (i = 0; i < UNINAME_MAX; i++)
-                        {
-                          int c2 = phase2_getc ();
-                          if (!(c2 >= ' ' && c2 <= '~'))
-                            {
-                              phase2_ungetc (c2);
-                              while (--i >= 0)
-                                phase2_ungetc (buf[i]);
-                              phase2_ungetc (c1);
-                              phase2_ungetc (c);
-                              ++*backslash_counter;
-                              return UNICODE ('\\');
-                            }
-                          if (c2 == '}')
-                            break;
-                          buf[i] = c2;
-                        }
-                      buf[i] = '\0';
-
-                      n = unicode_name_character ((char *) buf);
-                      if (n != UNINAME_INVALID)
-                        {
-                          *backslash_counter = 0;
-                          return UNICODE (n);
-                        }
-
-                      phase2_ungetc ('}');
-                      while (--i >= 0)
-                        phase2_ungetc (buf[i]);
-                    }
-                  phase2_ungetc (c1);
-                  phase2_ungetc (c);
-                  ++*backslash_counter;
-                  return UNICODE ('\\');
                 }
+              phase2_ungetc (c1);
+              phase2_ungetc (c);
+              ++*backslash_counter;
+              return UNICODE ('\\');
             }
         }
 
@@ -1165,8 +1198,49 @@ phase7_getuc (int quote_char,
 /* Combine characters into tokens.  Discard whitespace except newlines at
    the end of logical lines.  */
 
-/* Number of pending open parentheses/braces/brackets.  */
-static int open_pbb;
+/* Number of open f-strings f"...{ or f'...{ or f"""...{ or f'''...{ or
+   fr"...{ or fr'...{ or fr"""...{ or fr'''...{ */
+static int f_string_depth;
+
+/* Information per f-string nesting level.  */
+struct f_string_level
+{
+  /* Describes the start and end sequence of the f-string.
+     Only relevant for levels > 0.  */
+  int quote_char;
+  bool interpret_ansic;
+  bool triple;
+  /* Number of open '{' tokens.  */
+  int brace_depth;
+};
+
+/* Stack of f-string nesting levels.
+   The "current" element is f_string_stack[f_string_depth].  */
+static struct f_string_level *f_string_stack;
+/* Number of allocated elements in f_string_stack.  */
+static size_t f_string_stack_alloc;
+
+/* Adds a new f_string_stack level after f_string_depth was incremented.  */
+static void
+new_f_string_level (int quote_char, bool interpret_ansic, bool triple)
+{
+  if (f_string_depth == f_string_stack_alloc)
+    {
+      f_string_stack_alloc = 2 * f_string_stack_alloc + 1;
+      /* Now f_string_depth < f_string_stack_alloc.  */
+      f_string_stack =
+        (struct f_string_level *)
+        xrealloc (f_string_stack,
+                  f_string_stack_alloc * sizeof (struct f_string_level));
+    }
+  f_string_stack[f_string_depth].quote_char = quote_char;
+  f_string_stack[f_string_depth].interpret_ansic = interpret_ansic;
+  f_string_stack[f_string_depth].triple = triple;
+  f_string_stack[f_string_depth].brace_depth = 0;
+}
+
+/* Number of pending open parentheses/brackets.  */
+static int open_pb;
 
 static token_ty phase5_pushback[2];
 static int phase5_pushback_length;
@@ -1204,7 +1278,7 @@ phase5_get (token_ty *tp)
             savable_comment_reset ();
           /* Ignore newline if and only if it is used for implicit line
              joining.  */
-          if (open_pbb > 0)
+          if (open_pb > 0 || f_string_stack[f_string_depth].brace_depth > 0)
             continue;
           tp->type = token_type_other;
           return;
@@ -1226,13 +1300,13 @@ phase5_get (token_ty *tp)
               }
           }
           FALLTHROUGH;
-        case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
+        case 'A':           case 'C': case 'D': case 'E':
         case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
         case 'M': case 'N': case 'O': case 'P': case 'Q':
         case 'S': case 'T':           case 'V': case 'W': case 'X':
         case 'Y': case 'Z':
         case '_':
-        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
+        case 'a':           case 'c': case 'd': case 'e':
         case 'g': case 'h': case 'i': case 'j': case 'k': case 'l':
         case 'm': case 'n': case 'o': case 'p': case 'q':
         case 's': case 't':           case 'v': case 'w': case 'x':
@@ -1242,19 +1316,12 @@ phase5_get (token_ty *tp)
         symbol:
           /* Symbol, or part of a number.  */
           {
-            static char *buffer;
-            static int bufmax;
-            int bufpos;
+            struct string_buffer buffer;
 
-            bufpos = 0;
+            sb_init (&buffer);
             for (;;)
               {
-                if (bufpos >= bufmax)
-                  {
-                    bufmax = 2 * bufmax + 10;
-                    buffer = xrealloc (buffer, bufmax);
-                  }
-                buffer[bufpos++] = c;
+                sb_xappend1 (&buffer, c);
                 c = phase3_getc ();
                 switch (c)
                   {
@@ -1278,13 +1345,7 @@ phase5_get (token_ty *tp)
                   }
                 break;
               }
-            if (bufpos >= bufmax)
-              {
-                bufmax = 2 * bufmax + 10;
-                buffer = xrealloc (buffer, bufmax);
-              }
-            buffer[bufpos] = '\0';
-            tp->string = xstrdup (buffer);
+            tp->string = sb_xdupfree_c (&buffer);
             tp->type = token_type_symbol;
             return;
           }
@@ -1294,9 +1355,66 @@ phase5_get (token_ty *tp)
             int quote_char;
             bool interpret_ansic;
             bool interpret_unicode;
+            bool f_string;
             bool triple;
             unsigned int backslash_counter;
 
+            case 'B': case 'b':
+              {
+                int c1 = phase2_getc ();
+                if (c1 == '"' || c1 == '\'')
+                  {
+                    quote_char = c1;
+                    interpret_ansic = true;
+                    interpret_unicode = false;
+                    f_string = false;
+                    goto string;
+                  }
+                if (c1 == 'R' || c1 == 'r')
+                  {
+                    int c2 = phase2_getc ();
+                    if (c2 == '"' || c2 == '\'')
+                      {
+                        quote_char = c2;
+                        interpret_ansic = false;
+                        interpret_unicode = false;
+                        f_string = false;
+                        goto string;
+                      }
+                    phase2_ungetc (c2);
+                  }
+                phase2_ungetc (c1);
+                goto symbol;
+              }
+
+            case 'F': case 'f':
+              {
+                int c1 = phase2_getc ();
+                if (c1 == '"' || c1 == '\'')
+                  {
+                    quote_char = c1;
+                    interpret_ansic = true;
+                    interpret_unicode = true;
+                    f_string = true;
+                    goto string;
+                  }
+                if (c1 == 'R' || c1 == 'r')
+                  {
+                    int c2 = phase2_getc ();
+                    if (c2 == '"' || c2 == '\'')
+                      {
+                        quote_char = c2;
+                        interpret_ansic = false;
+                        interpret_unicode = true;
+                        f_string = true;
+                        goto string;
+                      }
+                    phase2_ungetc (c2);
+                  }
+                phase2_ungetc (c1);
+                goto symbol;
+              }
+
             case 'R': case 'r':
               {
                 int c1 = phase2_getc ();
@@ -1304,9 +1422,23 @@ phase5_get (token_ty *tp)
                   {
                     quote_char = c1;
                     interpret_ansic = false;
-                    interpret_unicode = false;
+                    interpret_unicode = true;
+                    f_string = false;
                     goto string;
                   }
+                if (c1 == 'F' || c1 == 'f')
+                  {
+                    int c2 = phase2_getc ();
+                    if (c2 == '"' || c2 == '\'')
+                      {
+                        quote_char = c2;
+                        interpret_ansic = false;
+                        interpret_unicode = true;
+                        f_string = true;
+                        goto string;
+                      }
+                    phase2_ungetc (c2);
+                  }
                 phase2_ungetc (c1);
                 goto symbol;
               }
@@ -1319,20 +1451,9 @@ phase5_get (token_ty *tp)
                     quote_char = c1;
                     interpret_ansic = true;
                     interpret_unicode = true;
+                    f_string = false;
                     goto string;
                   }
-                if (c1 == 'R' || c1 == 'r')
-                  {
-                    int c2 = phase2_getc ();
-                    if (c2 == '"' || c2 == '\'')
-                      {
-                        quote_char = c2;
-                        interpret_ansic = false;
-                        interpret_unicode = true;
-                        goto string;
-                      }
-                    phase2_ungetc (c2);
-                  }
                 phase2_ungetc (c1);
                 goto symbol;
               }
@@ -1340,7 +1461,8 @@ phase5_get (token_ty *tp)
             case '"': case '\'':
               quote_char = c;
               interpret_ansic = true;
-              interpret_unicode = false;
+              interpret_unicode = true;
+              f_string = false;
             string:
               triple = false;
               lexical_context = lc_string;
@@ -1370,13 +1492,28 @@ phase5_get (token_ty *tp)
                 for (;;)
                   {
                     int uc = phase7_getuc (quote_char, triple, interpret_ansic,
-                                           interpret_unicode, &backslash_counter);
+                                           interpret_unicode, f_string,
+                                           &backslash_counter);
 
                     /* Keep line_number in sync.  */
                     msb.line_number = line_number;
 
                     if (uc == P7_EOF || uc == P7_STRING_END)
-                      break;
+                      {
+                        tp->mixed_string = mixed_string_buffer_result (&msb);
+                        tp->comment = add_reference (savable_comment);
+                        tp->type = (f_string ? token_type_498 : token_type_string);
+                        break;
+                      }
+
+                    if (uc == P7_498_START_OF_EXPRESSION) /* implies f_string */
+                      {
+                        mixed_string_buffer_destroy (&msb);
+                        tp->type = token_type_l498;
+                        f_string_depth++;
+                        new_f_string_level (quote_char, interpret_ansic, triple);
+                        break;
+                      }
 
                     if (IS_UNICODE (uc))
                       {
@@ -1388,22 +1525,58 @@ phase5_get (token_ty *tp)
                     else
                       mixed_string_buffer_append_char (&msb, uc);
                   }
-                tp->mixed_string = mixed_string_buffer_result (&msb);
-                tp->comment = add_reference (savable_comment);
                 lexical_context = lc_outside;
-                tp->type = token_type_string;
               }
               return;
           }
 
+        case '{':
+          f_string_stack[f_string_depth].brace_depth++;
+          tp->type = token_type_other;
+          return;
+
+        case '}':
+          if (f_string_stack[f_string_depth].brace_depth > 0)
+            f_string_stack[f_string_depth].brace_depth--;
+          else if (f_string_depth > 0)
+            {
+              /* Middle or right part of f-string.  */
+              int quote_char = f_string_stack[f_string_depth].quote_char;
+              bool interpret_ansic = f_string_stack[f_string_depth].interpret_ansic;
+              bool triple = f_string_stack[f_string_depth].triple;
+              unsigned int backslash_counter = 0;
+              for (;;)
+                {
+                  int uc = phase7_getuc (quote_char, triple, interpret_ansic,
+                                         true, true,
+                                         &backslash_counter);
+
+                  if (uc == P7_EOF || uc == P7_STRING_END)
+                    {
+                      tp->type = token_type_r498;
+                      f_string_depth--;
+                      break;
+                    }
+
+                  if (uc == P7_498_START_OF_EXPRESSION)
+                    {
+                      tp->type = token_type_m498;
+                      break;
+                    }
+                }
+              return;
+            }
+          tp->type = token_type_other;
+          return;
+
         case '(':
-          open_pbb++;
+          open_pb++;
           tp->type = token_type_lparen;
           return;
 
         case ')':
-          if (open_pbb > 0)
-            open_pbb--;
+          if (open_pb > 0)
+            open_pb--;
           tp->type = token_type_rparen;
           return;
 
@@ -1411,15 +1584,15 @@ phase5_get (token_ty *tp)
           tp->type = token_type_comma;
           return;
 
-        case '[': case '{':
-          open_pbb++;
-          tp->type = (c == '[' ? token_type_lbracket : token_type_other);
+        case '[':
+          open_pb++;
+          tp->type = token_type_lbracket;
           return;
 
-        case ']': case '}':
-          if (open_pbb > 0)
-            open_pbb--;
-          tp->type = (c == ']' ? token_type_rbracket : token_type_other);
+        case ']':
+          if (open_pb > 0)
+            open_pb--;
+          tp->type = token_type_rbracket;
           return;
 
         case '+':
@@ -1457,7 +1630,7 @@ static void
 x_python_lex (token_ty *tp)
 {
   phase5_get (tp);
-  if (tp->type == token_type_string)
+  if (tp->type == token_type_string || tp->type == token_type_498)
     {
       mixed_string_ty *sum = tp->mixed_string;
 
@@ -1473,7 +1646,8 @@ x_python_lex (token_ty *tp)
             case token_type_plus:
               {
                 phase5_get (&token3);
-                if (token3.type == token_type_string)
+                if (token3.type == token_type_string
+                    || token3.type == token_type_498)
                   {
                     free_token (&token2);
                     tp2 = &token3;
@@ -1483,6 +1657,7 @@ x_python_lex (token_ty *tp)
               }
               break;
             case token_type_string:
+            case token_type_498:
               tp2 = &token2;
               break;
             default:
@@ -1511,6 +1686,14 @@ x_python_lex (token_ty *tp)
 static flag_context_list_table_ty *flag_context_list_table;
 
 
+/* Maximum supported nesting depth.  */
+#define MAX_NESTING_DEPTH 1000
+
+/* Current nesting depths.  */
+static int paren_nesting_depth;
+static int bracket_nesting_depth;
+
+
 /* The file is broken into tokens.  Scan the token stream, looking for
    a keyword, followed by a left paren, followed by a string.  When we
    see this sequence, we have something to remember.  We assume we are
@@ -1534,7 +1717,7 @@ static flag_context_list_table_ty *flag_context_list_table;
 static bool
 extract_balanced (message_list_ty *mlp,
                   token_type_ty delim,
-                  flag_context_ty outer_context,
+                  flag_region_ty *outer_region,
                   flag_context_list_iterator_ty context_iter,
                   struct arglist_parser *argparser)
 {
@@ -1547,9 +1730,9 @@ extract_balanced (message_list_ty *mlp,
   /* Context iterator that will be used if the next token is a '('.  */
   flag_context_list_iterator_ty next_context_iter =
     passthrough_context_list_iterator;
-  /* Current context.  */
-  flag_context_ty inner_context =
-    inherited_context (outer_context,
+  /* Current region.  */
+  flag_region_ty *inner_region =
+    inheriting_region (outer_region,
                        flag_context_list_iterator_advance (&context_iter));
 
   /* Start state is 0.  */
@@ -1585,14 +1768,20 @@ extract_balanced (message_list_ty *mlp,
           continue;
 
         case token_type_lparen:
+          if (++paren_nesting_depth > MAX_NESTING_DEPTH)
+            if_error (IF_SEVERITY_FATAL_ERROR,
+                      logical_file_name, line_number, (size_t)(-1), false,
+                      _("too many open parentheses"));
           if (extract_balanced (mlp, token_type_rparen,
-                                inner_context, next_context_iter,
+                                inner_region, next_context_iter,
                                 arglist_parser_alloc (mlp,
                                                       state ? next_shapes : NULL)))
             {
               arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
               return true;
             }
+          paren_nesting_depth--;
           next_context_iter = null_context_list_iterator;
           state = 0;
           continue;
@@ -1601,6 +1790,7 @@ extract_balanced (message_list_ty *mlp,
           if (delim == token_type_rparen || delim == token_type_eof)
             {
               arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
               return false;
             }
           next_context_iter = null_context_list_iterator;
@@ -1609,8 +1799,9 @@ extract_balanced (message_list_ty *mlp,
 
         case token_type_comma:
           arg++;
-          inner_context =
-            inherited_context (outer_context,
+          unref_region (inner_region);
+          inner_region =
+            inheriting_region (outer_region,
                                flag_context_list_iterator_advance (
                                  &context_iter));
           next_context_iter = passthrough_context_list_iterator;
@@ -1618,13 +1809,20 @@ extract_balanced (message_list_ty *mlp,
           continue;
 
         case token_type_lbracket:
+          if (++bracket_nesting_depth > MAX_NESTING_DEPTH)
+            if_error (IF_SEVERITY_FATAL_ERROR,
+                      logical_file_name, line_number, (size_t)(-1), false,
+                      _("too many open brackets"));
           if (extract_balanced (mlp, token_type_rbracket,
-                                null_context, null_context_list_iterator,
+                                null_context_region (),
+                                null_context_list_iterator,
                                 arglist_parser_alloc (mlp, NULL)))
             {
               arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
               return true;
             }
+          bracket_nesting_depth--;
           next_context_iter = null_context_list_iterator;
           state = 0;
           continue;
@@ -1633,6 +1831,7 @@ extract_balanced (message_list_ty *mlp,
           if (delim == token_type_rbracket || delim == token_type_eof)
             {
               arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
               return false;
             }
           next_context_iter = null_context_list_iterator;
@@ -1640,6 +1839,7 @@ extract_balanced (message_list_ty *mlp,
           continue;
 
         case token_type_string:
+        case token_type_498:
           {
             lex_pos_ty pos;
 
@@ -1651,12 +1851,12 @@ extract_balanced (message_list_ty *mlp,
                 char *string = mixed_string_contents (token.mixed_string);
                 mixed_string_free (token.mixed_string);
                 remember_a_message (mlp, NULL, string, true, false,
-                                    inner_context, &pos,
+                                    inner_region, &pos,
                                     NULL, token.comment, true);
               }
             else
               arglist_parser_remember (argparser, arg, token.mixed_string,
-                                       inner_context,
+                                       inner_region,
                                        pos.file_name, pos.line_number,
                                        token.comment, true);
           }
@@ -1667,8 +1867,12 @@ extract_balanced (message_list_ty *mlp,
 
         case token_type_eof:
           arglist_parser_done (argparser, arg);
+          unref_region (inner_region);
           return true;
 
+        case token_type_l498:
+        case token_type_m498:
+        case token_type_r498:
         case token_type_plus:
         case token_type_other:
           next_context_iter = null_context_list_iterator;
@@ -1720,18 +1924,23 @@ extract_python (FILE *f,
 
   continuation_or_nonblank_line = false;
 
-  open_pbb = 0;
+  open_pb = 0;
 
   phase5_pushback_length = 0;
 
+  f_string_depth = 0;
+  new_f_string_level (0, false, false);
+
   flag_context_list_table = flag_table;
+  paren_nesting_depth = 0;
+  bracket_nesting_depth = 0;
 
   init_keywords ();
 
   /* Eat tokens until eof is seen.  When extract_balanced returns
      due to an unbalanced closing parenthesis, just restart it.  */
   while (!extract_balanced (mlp, token_type_eof,
-                            null_context, null_context_list_iterator,
+                            null_context_region (), null_context_list_iterator,
                             arglist_parser_alloc (mlp, NULL)))
     ;
 
